package cz.fidentis.analyst.mesh.core;

import org.testng.annotations.Test;

import static org.testng.AssertJUnit.assertEquals;

/**
 * Unit tests for CornerTable
 */
public class CornerTableTest {

    /**
     * Unit test for getIndexOfFace
     */
    @Test
    public void getIndexOfFace() {
        CornerTable table = new CornerTable();
        for (int i = 0; i < 10; i++) {
            table.addRow(new CornerTableRow(i, -1));
        }

        for (int i = 0; i < 10; i++) {
            assertEquals(i / 3, table.getIndexOfFace(i));
        }
    }

    /**
     * Unit test for getIndexOfFaceNegativeIndex with negative index
     */
    @Test
    public void getIndexOfFaceNegativeIndex() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        assertEquals(-2, table.getIndexOfFace(-1));
    }

    /**
     * Unit test for getIndexOfFaceNegativeIndex with index out of range
     */
    @Test
    public void getIndexOfFaceIndexOutOfRange() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        assertEquals(-2, table.getIndexOfFace(2));
    }

    /**
     * Unit test for getIndexOfOppositeCorner
     */
    @Test
    public void getIndexOfOppositeCorner() {
        CornerTable table = new CornerTable();
        for (int i = 0; i < 10; i++) {
            table.addRow(new CornerTableRow(i, i - 1));
        }

        for (int i = 1; i < 10; i++) {
            assertEquals(i - 1, table.getIndexOfOppositeCorner(i));
        }
    }

    /**
     * Unit test for getIndexOfOppositeCorner without opposite corner
     */
    @Test
    public void getIndexOfOppositeCornerNoOppositeCorner() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        assertEquals(-1, table.getIndexOfOppositeCorner(0));
    }

    /**
     * Unit test for getIndexOfOppositeCorner with negative index of corner
     */
    @Test
    public void getIndexOfOppositeCornerNegativeIndex() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        assertEquals(-2, table.getIndexOfOppositeCorner(-1));
    }

    /**
     * Unit test for getIndexOfOppositeCorner with index of corner out of range
     */
    @Test
    public void getIndexOfOppositeCornerOutOfRange() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        assertEquals(-2, table.getIndexOfOppositeCorner(2));
    }

    /**
     * Unit test for getIndexOfNextCornerInFace
     */
    @Test
    public void getIndexOfNextCornerInFaceFirstTriangle() {
        CornerTable table = new CornerTable();
        for (int i = 0; i < 9; i++) {
            table.addRow(new CornerTableRow(i, -1));
        }

        assertEquals(1, table.getIndexOfNextCornerInFace(0));
        assertEquals(2, table.getIndexOfNextCornerInFace(1));
        assertEquals(0, table.getIndexOfNextCornerInFace(2));
    }

    /**
     * Unit test for getIndexOfNextCornerInFace
     */
    @Test
    public void getIndexOfNextCornerInFaceMiddleTriangle() {
        CornerTable table = new CornerTable();
        for (int i = 0; i < 9; i++) {
            table.addRow(new CornerTableRow(i, -1));
        }

        assertEquals(4, table.getIndexOfNextCornerInFace(3));
        assertEquals(5, table.getIndexOfNextCornerInFace(4));
        assertEquals(3, table.getIndexOfNextCornerInFace(5));
    }

    /**
     * Unit test for getIndexOfNextCornerInFace
     */
    @Test
    public void getIndexOfNextCornerInFaceLastTriangle() {
        CornerTable table = new CornerTable();
        for (int i = 0; i < 9; i++) {
            table.addRow(new CornerTableRow(i, -1));
        }

        assertEquals(7, table.getIndexOfNextCornerInFace(6));
        assertEquals(8, table.getIndexOfNextCornerInFace(7));
        assertEquals(6, table.getIndexOfNextCornerInFace(8));
    }

    /**
     * Unit test for getIndexOfNextCornerInFace with negative index of corner
     */
    @Test
    public void getIndexOfNextCornerInFaceNegativeIndex() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        assertEquals(-2, table.getIndexOfNextCornerInFace(-1));
    }

    /**
     * Unit test for getIndexOfNextCornerInFace with index of corner out of range
     */
    @Test
    public void getIndexOfNextCornerInFaceOutOfRange() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        assertEquals(-2, table.getIndexOfNextCornerInFace(2));
    }

    /**
     * Unit test for getIndexOfPreviousCornerInFace
     */
    @Test
    public void getIndexOfPreviousCornerInFaceFirstTriangle() {
        CornerTable table = new CornerTable();
        for (int i = 0; i < 9; i++) {
            table.addRow(new CornerTableRow(i, -1));
        }

        assertEquals(2, table.getIndexOfPreviousCornerInFace(0));
        assertEquals(0, table.getIndexOfPreviousCornerInFace(1));
        assertEquals(1, table.getIndexOfPreviousCornerInFace(2));
    }

    /**
     * Unit test for getIndexOfPreviousCornerInFace
     */
    @Test
    public void getIndexOfPreviousCornerInFaceMiddleTriangle() {
        CornerTable table = new CornerTable();
        for (int i = 0; i < 9; i++) {
            table.addRow(new CornerTableRow(i, -1));
        }

        assertEquals(5, table.getIndexOfPreviousCornerInFace(3));
        assertEquals(3, table.getIndexOfPreviousCornerInFace(4));
        assertEquals(4, table.getIndexOfPreviousCornerInFace(5));
    }

    /**
     * Unit test for getIndexOfPreviousCornerInFace
     */
    @Test
    public void getIndexOfPreviousCornerInFaceLastTriangle() {
        CornerTable table = new CornerTable();
        for (int i = 0; i < 9; i++) {
            table.addRow(new CornerTableRow(i, -1));
        }

        assertEquals(8, table.getIndexOfPreviousCornerInFace(6));
        assertEquals(6, table.getIndexOfPreviousCornerInFace(7));
        assertEquals(7, table.getIndexOfPreviousCornerInFace(8));
    }

    /**
     * Unit test for getIndexOfPreviousCornerInFace with negative index of corner
     */
    @Test
    public void getIndexOfPreviousCornerInFaceNegativeIndex() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        assertEquals(-2, table.getIndexOfPreviousCornerInFace(-1));
    }

    /**
     * Unit test for getIndexOfPreviousCornerInFace with index of corner out of range
     */
    @Test
    public void getIndexOfPreviousCornerInFaceOutOfRange() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        assertEquals(-2, table.getIndexOfPreviousCornerInFace(2));
    }

    /**
     * Unit test for getIndexOfTipCornerOnLeft
     */
    @Test
    public void getIndexOfTipCornerOnLeft() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(2, 5));
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(3, 2));

        assertEquals(5, table.getIndexOfTipCornerOnLeft(0));
    }

    /**
     * Unit test for getIndexOfTipCornerOnLeft with no left corner
     */
    @Test
    public void getIndexOfTipCornerOnLeftNoLeftCorner() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(2, 5));
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(3, 2));

        assertEquals(-1, table.getIndexOfTipCornerOnLeft(1));
    }

    /**
     * Unit test for getIndexOfTipCornerOnLeft with negative index of corner
     */
    @Test
    public void getIndexOfTipCornerOnLeftNegativeIndex() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(2, 5));
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(3, 2));

        assertEquals(-2, table.getIndexOfTipCornerOnLeft(-1));
    }

    /**
     * Unit test for getIndexOfTipCornerOnLeft with index of corner out of range
     */
    @Test
    public void getIndexOfTipCornerOnLeftOutOfRange() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(2, 5));
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(3, 2));

        assertEquals(-2, table.getIndexOfTipCornerOnLeft(6));
    }

    /**
     * Unit test for getIndexOfTipCornerOnRight
     */
    @Test
    public void getIndexOfTipCornerOnRight() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(2, 5));
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(3, 2));

        assertEquals(5, table.getIndexOfTipCornerOnRight(1));
    }

    /**
     * Unit test for getIndexOfTipCornerOnRight with no right corner
     */
    @Test
    public void getIndexOfTipCornerOnLeftNoRightCorner() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(2, 5));
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(3, 2));

        assertEquals(-1, table.getIndexOfTipCornerOnRight(0));
    }

    /**
     * Unit test for getIndexOfTipCornerOnRight with negative index of corner
     */
    @Test
    public void getIndexOfTipCornerOnRightNegativeIndex() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(2, 5));
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(3, 2));

        assertEquals(-2, table.getIndexOfTipCornerOnRight(-1));
    }

    /**
     * Unit test for getIndexOfTipCornerOnRight with index of corner out of range
     */
    @Test
    public void getIndexOfTipCornerOnRightOutOfRange() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(2, 5));
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(3, 2));

        assertEquals(-2, table.getIndexOfTipCornerOnLeft(6));
    }

    /**
     * Unit test for getNextAroundCorner
     */
    @Test
    public void getNextAroundCorner() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(2, 5));
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(3, 2));

        assertEquals(3, table.getNextAroundCorner(0));
    }

    /**
     * Unit test for getNextAroundCorner with no corner on around position
     */
    @Test
    public void getNextAroundCornerNoAroundCorner() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(2, 5));
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(3, 2));

        assertEquals(-1, table.getNextAroundCorner(1));
    }

    /**
     * Unit test for getNextAroundCorner with negative index of corner
     */
    @Test
    public void getNextAroundCornerNegativeIndex() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(2, 5));
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(3, 2));

        assertEquals(-2, table.getNextAroundCorner(-1));
    }

    /**
     * Unit test for getNextAroundCorner with index of corner out of range
     */
    @Test
    public void getNextAroundCornerOutOfRange() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(2, 5));
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(3, 2));

        assertEquals(-2, table.getNextAroundCorner(6));
    }

    /**
     * Unit test for addRow
     */
    @Test
    public void addRow() {
        CornerTable table = new CornerTable();
        assertEquals(0, table.getSize());
        table.addRow(new CornerTableRow(0, -1));
        assertEquals(1, table.getSize());
    }

    /**
     * Unit test for replaceRow
     */
    @Test
    public void replaceRow() {
        CornerTable table = new CornerTable();
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(2, 5));
        table.addRow(new CornerTableRow(0, -1));
        table.addRow(new CornerTableRow(1, -1));
        table.addRow(new CornerTableRow(3, 2));

        assertEquals(6, table.getSize());

        table.replaceRow(1, new CornerTableRow(table.getRow(1).getVertexIndex(), 42));
        assertEquals(6, table.getSize());
        assertEquals(42, table.getRow(1).getOppositeCornerIndex());
    }

    /**
     * Unit test for getSize
     */
    @Test
    public void getSize() {
        CornerTable table = new CornerTable();

        assertEquals(0, table.getSize());

        for (int i = 0; i < 9; i++) {
            table.addRow(new CornerTableRow(i, -1));
        }

        assertEquals(9, table.getSize());
    }

    /**
     * Unit test for getRow
     */
    @Test
    public void getRow() {
        CornerTable table = new CornerTable();
        for (int i = 0; i < 9; i++) {
            table.addRow(new CornerTableRow(i, -1));
        }

        for (int i = 0; i < 9; i++) {
            assertEquals(i, table.getRow(i).getVertexIndex());
        }
    }
}